###############################################################################
# parse_conf.pm
#
# Version: 0.1
# by Joffrey Fitz (fitzj@informatik.uni-tuebingen.de)
# created Wed Sep  6 12:07:54 CEST 2006
#
# last modified: 
#
# This program is free software; you can redistribute it and/or modify it                        
# under the terms of the GNU General Public License as published by the Free                     
# Software Foundation; either version 2, or (at your option) any later 
# version.
#
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
# or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
# for more details.
# 
# Synopsis:
#
# use parse_conf;
# 
# parsefile($ARGV[0]);
# checkconf();
# 
# print $dbname, $host, $port, $password, "\n";
# print $div_threshold, $right_neigbh_threshold, $left_neigbh_threshold, "\n";
#
###############################################################################

package      parse_conf;
require      Exporter;
use 		 Switch;

our @ISA       = qw(Exporter);
our @EXPORT    = qw(parsefile checkconf $dbname $host $port $password $div_threshold $right_neigbh_threshold $left_neigbh_threshold);
our @EXPORT_OK = qw($weight);
our $VERSION   = 1.00;

#############################################################################

#my ($dbname, $host, $port, $password);
#my ($div_threshold, $right_neigbh_threshold, $left_neigbh_threshold);
	
sub parsefile {
	my $conf_file = shift;

	open(FH, "$conf_file") || die $!;
	
	while(<FH>) {
		next if(/^#/);
		chomp;
		switch($_) {
			case /^DBNAME=.*$/		{ $dbname = exval($_); }
			case /^DBHOST=.*$/		{ $host = exval($_); }
			case /^DBPORT=.*$/		{ $port = exval($_); }
			case /^DBPASS=.*$/		{ $password = exval($_); }
			case /^DIVTHRESHOLD=.*$/				{ $div_threshold = exval($_); }
			case /^RIGHTNEIGBHTHRESHOLD=.*$/		{ $right_neigbh_threshold = exval($_); }
			case /^LEFTNEIGHBHTHRESHOLD=.*$/		{ $left_neigbh_threshold = exval($_); }
			else					{}
		}
	}
}

# extract value from config file
# config files contains shell assignments like DBNAME="foobar"
# exval returns foobar
sub exval {
	my $res;
	($res = $_) =~ s/^(.*?)"(.*?)"(.*)$/$2/;
	return $res;
}

# check configuration
sub checkconf {
	# set threshold to unlimited (>1) if it is not defined
  	$div_threshold = 1.1 if($div_threshold eq "");
  	__die("RIGHTNEIGBHTHRESHOLD") if($right_neigbh_threshold eq "");
	__die("LEFTNEIGHBHTHRESHOLD") if($left_neigbh_threshold eq "");

	# check database parameters
	__die("DBNAME") if($dbname eq "");
	__die("DBHOST") if($host eq "");
	__die("DBPORT") if($port eq "");
	# password for database connection is optional
	#__die("No value given for parameter DBNAME") if($dbname eq "");
}

sub __die {
	my $msg = shift;
	die "!! No value given for parameter " . $msg . ".\nCheck your configuration.\n";
}

1;