#!/usr/bin/perl -w

#
# msqt-sniped.cgi
#
# Author: Joffrey Fitz <fitzj@informatik.uni-tuebingen.de>
#
# Copyright (c) 2007 Max Planck Institute for Developmental Biology, 
# 	http://www.eb.tuebingen.mpg.de
#
# This program is part of the MSQT project by Norman Warthmann, Joffrey Fitz 
# and Detlef Weigel (http://msqt.weigelworld.org)
#
# This program is free software; you can redistribute it and/or modify it                        
# under the terms of the GNU General Public License as published by the Free                     
# Software Foundation; either version 2, or (at your option) any later 
# version.
#
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
# or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
# for more details.
#

use strict;
use CGI;
use CGI::Carp qw( fatalsToBrowser );
use DBI;
use LWP::Simple;

# global
my $DEBUG = 0;
# CGI
my $script_loc = "[msqtcgiroot]/msqt-sniped.cgi";

# GUI templates
my $content_file = "html/tmpl_msqt-sniped.html";

# Fasta files
my $fasta_base_url = "[msqtwebroot]/fasta_files";
# Jalview
my $alivie_archive = "[msqtwebroot]/msqt/alivie.jar";
# mstq-e
my $msqt_e_loc = "[msqtcgiroot]/msqt-e.cgi";
my $error_file = "html/server_down_error.html";


# database config
my $dbname = "[DBNAME]";
my $host = "[DBHOST]";
my $port = "[DBPORT]";
my $options = "";
my $username = "msqt";
my $password = "[DBPASS]";

my $dbh;
my $sth;
my $row;
my $query;

# default values
my $lnhl_default = 15;
my $rnhl_default = 15;
my $diff_value_default = 0.5;
my $sum_value_default = 0.8;

my $html_results;

####################################################################


# do not buffer
$| = 1;



$dbh = DBI->connect("dbi:Pg:dbname=$dbname;host=$host;port=$port;".
	"", "$username", "$password",
        {AutoCommit => 0} ) || db_connect_error();

my $q = new CGI;

my $file_buf;
open FORM, $content_file or die $!;
# read template
while(<FORM>) {
	$file_buf .= $_;
}



if($q->param( "get" )) {
	msqt_sniped();
}
else {
	# print page only
	print "Content-type: text/html\n\n";
	
	# generate schema combobox
	# query existing schemas from database
	my $combobox_buf;
	my $combobox_options;
	my $query = "SELECT msqt_schema_name, default_dataset FROM public.msqt_config";
	if($DEBUG) { print "query: $query<br>"; }
	$sth = $dbh->prepare( $query );
	$sth->execute() || print "<font color=red><b>" . $sth->errstr . "</b></font><br><br>";

	# generate options for combobox
	my $row_cnter=0;
	my @schema_row_arr;
	while ( @schema_row_arr = $sth->fetchrow_array ) {
		$combobox_options .= "<option value=\"" . $schema_row_arr[0] . "\"";
		$combobox_options .= " selected" if($schema_row_arr[1] == 1); # Default dataset?
		$combobox_options .= ">$schema_row_arr[0]</option>\n";
		$row_cnter++;
	}
	$sth->finish();

	# Insert headline
	$file_buf =~ s/--headline--/MSQT\/<b>SNIPED!<\/b>/;
	
	# Insert scriptloc into FORM tag
	$file_buf =~ s/--MSQT-TEMPL:scriptloc--/$script_loc/;
	

	# Generate tabbar
	my $tabbar_buf;
	$tabbar_buf = <<HTML;
	<div class="tabbar_table">
			<div class="tabbar_tr">
				<div class="tabbar_td"><a class="tabbar_link" href="[msqtwebroot]/index.html">Home</a></div> <span class="toolbar_divider">&nbsp;&nbsp;&nbsp;&nbsp;</span>
				<div class="tabbar_td"><a class="tabbar_link" href="[msqtcgiroot]/msqt-sbe.cgi">SNPs Between Ecotypes</a></div> <span class="toolbar_divider">&nbsp;&nbsp;&nbsp;&nbsp;</span>
				<div class="tabbar_td"><a class="tabbar_link" href="[msqtcgiroot]/msqt-adf.cgi">Assay Development Formater</a></div> <span class="toolbar_divider">&nbsp;&nbsp;&nbsp;&nbsp;</span>
				<div class="tabbar_td"><a class="tabbar_link" href="[msqtcgiroot]/msqt-e.cgi">Expert Mode</a></div> <span class="toolbar_divider">&nbsp;&nbsp;&nbsp;&nbsp;</span>
				<div class="tabbar_td"><a class="tabbar_link" href="[msqtcgiroot]/msqt-sniped.cgi">SNIPED!</a></div> <span class="toolbar_divider">&nbsp;&nbsp;&nbsp;&nbsp;</span>
				<div class="tabbar_td"><a class="tabbar_link" href="[msqtcgiroot]/msqtadmin/msqtadmin.cgi" target="__msqtadmin__">Admin</a></div> <span class="toolbar_divider">&nbsp;&nbsp;&nbsp;&nbsp;</span>
				<div class="tabbar_td"><a class="tabbar_link" href="[msqtwebroot]/doc.html">Documentation</a></div> <span class="toolbar_divider">&nbsp;&nbsp;&nbsp;&nbsp;</span>
				<div class="tabbar_td"><a class="tabbar_link" href="[msqtwebroot]/about.html">About</a></div>
			</div>
		</div>
HTML
	$file_buf =~ s/<!--MSQT-TEMPL:tabbar-->/$tabbar_buf/;
	
	# generate combobox
	$combobox_buf = "<select name=\"dbschema\" tabindex=\"1\">";
	$combobox_buf .= $combobox_options;
	$combobox_buf .= "</select>";

	# insert widgets into template
	$file_buf =~ s/--schema_list--/$combobox_buf/;
	$file_buf =~ s/--diff_value_default--/$diff_value_default/;
	$file_buf =~ s/--sum_value_default--/$sum_value_default/;
	$file_buf =~ s/--lnhl_default--/$lnhl_default/;
	$file_buf =~ s/--rnhl_default--/$rnhl_default/;
	print $file_buf;
	
}

####################################################################


$dbh->disconnect || die;
exit 0;



####################################################################


sub msqt_sniped {
    my $schema_type;

    # get parameters
	
	# db schema
	my $dbschema = $q->param( "dbschema" );
	# operands
	my $op_diff_value = $q->param( "op_diff_value" );
	my $op_sum_value = $q->param( "op_sum_value" );
	my $op_lnhl = $q->param( "op_lnhl" );
	my $op_rnhl = $q->param( "op_rnhl" );
	# concatenations
	my $con_values = $q->param( "con_values" );
	my $con_nh = $q->param( "con_nh" );
	# input fields
	my $diff_value = $q->param( "diff_value" );
	my $sum_value = $q->param( "sum_value" );
	my $lnhl = $q->param( "lnhl" );
	my $rnhl = $q->param( "rnhl" );


	# Determine which database structure is used
    $query = "SELECT msqt_schema_type FROM public.msqt_config WHERE msqt_schema_name = '$dbschema'";
    if($DEBUG) { print "query: $query<br>"; }

	$sth = $dbh->prepare( $query );
	$sth->execute() || print "<font color=red><b>" . $sth->errstr . "</b></font><br><br>";
	while ( $row = $sth->fetchrow ) {
		$schema_type = $row;
    }
    #
    #
          
	$file_buf =~ s/--diff_value_default--/$diff_value/;
	$file_buf =~ s/--sum_value_default--/$sum_value/;
	$file_buf =~ s/--lnhl_default--/$lnhl/;
	$file_buf =~ s/--rnhl_default--/$rnhl/;

	

	# Determine which database structure is used
	#$query = "SELECT msqt_schema_type FROM public.msqt_config WHERE msqt_schema_name = '$dbtable'";
	#$sth = $dbh->prepare( $query );
	#$sth->execute() || print "<font color=red><b>" . $sth->errstr . "</b></font><br><br>";
	#while ( $row = $sth->fetchrow ) {
	#  $schema_type = $row;
	#}

if($schema_type eq "struct_position_chromosome") {

	$query = <<EOQ;
-- This is a query for '$dbschema' dataset
SELECT * FROM $dbschema.sniped WHERE
( (freq_diff $op_diff_value $diff_value) $con_values (freq_sum $op_sum_value $sum_value) ) 
AND
( (neighborhood_left_len $op_lnhl $lnhl) $con_nh (neighborhood_right_len $op_rnhl $rnhl) )

-- Uncomment one or more of the following lines
-- to refine your query
--
-- Limit results to chromosome 1
-- AND chrom = '1'
--
-- A must have ecotype
-- AND ( (ecot1 LIKE '%Lz-0%') OR (ecot2 LIKE '%Lz-0%') )

ORDER by chrom, fragid
;

EOQ

}
elsif($schema_type eq "struct_position_name") {

	$query = <<EOQ;
-- This is a query for '$dbschema' dataset
SELECT * FROM $dbschema.sniped WHERE
( (freq_diff $op_diff_value $diff_value) $con_values (freq_sum $op_sum_value $sum_value) ) 
AND
( (neighborhood_left_len $op_lnhl $lnhl) $con_nh (neighborhood_right_len $op_rnhl $rnhl) )

-- Uncomment one or more of the following lines
-- to refine your query
--
-- A must have ecotype
-- AND ( (ecot1 LIKE '%Lz-0%') OR (ecot2 LIKE '%Lz-0%') )

ORDER by fragid
;

EOQ

}


#	execute_sql();
#	my $html_buf = $html_results;
# 	$file_buf =~ s/<!--results-->/$html_buf/;
# 	print $file_buf;
 	close FORM;
	
	call_msqt_e();
}


# called by parse_params()
sub execute_sql {
	if($DEBUG) { print "query: $query<br>"; }
	$sth = $dbh->prepare( $query );# || 
		#$html_results = "<font color=red><b>" . $sth->errstr . "</b></font><br><br>";

	if (not $sth->execute()) {
		$html_results = "<font color=red><b>" . $sth->errstr . "</b></font><br><br>";
		return;
	}
	
	my @col_headers = @{$sth-> {NAME_uc}}; # Now you have the field names in @Header.
	my $col_num = scalar(@col_headers);
	
	$html_results .= "<table style=\"font-family:Courier,fixed:\" border=0 width=100%>";
	$html_results .= "<tr bgcolor=\"lightgrey\">";
	foreach my $td (@col_headers) {
			$html_results .= "<td><b>$td</b>     </td>\n";
	}
	$html_results .= "</tr>";
	# generate options for listboxes
	while ( my @row_arr = $sth->fetchrow_array ) {
		$html_results .= "<tr>";
		foreach my $td (@row_arr) {
			$html_results .= "<td>$td</td>\n";
		}
		$html_results .= "</tr>";
	}
	$html_results .= "</table>";
	$sth->finish();
	
	$html_results .= "<br><p>ready.";

}


sub call_msqt_e {
	$query = "--\n-- This query was generated by MSQT/SNIPED!\n--\n\n".$query;
	$query =~ s/([^A-Za-z0-9])/sprintf("%%%02X", ord($1))/seg;
	my $msqt_e_params = "exec_sql_query=Execute+SQL+Query&usr_sql_query=$query";
#my $content = get "$msqt_e_loc$msqt_e_params";
#print $content;
#print "Location $msqt_e_loc$msqt_e_params\n\n";
#print "Content-type: text/html\n\n";

#$q = new CGI;
#print $q->redirect( "$msqt_e_loc$msqt_e_params" );
	
# url encode query
	
	print "Location: $msqt_e_loc?$msqt_e_params\n\n";
}

sub db_connect_error {
	open FORM, $error_file or die $!;
	# read template
	print "Content-type: text/html\n\n";
	while(<FORM>) {
		print $_;
	}
	exit(0);
}

