package SQLQuery2HTMLTable;

#
# SQLQuery2HTMLTable.pm
#
# (c) 2006 by Joffrey Fitz <fitzj@informatik.uni-tuebingen.de>
#
# This program is free software; you can redistribute it and/or modify it                        
# under the terms of the GNU General Public License as published by the Free                     
# Software Foundation; either version 2, or (at your option) any later 
# version.
#
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
# or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
# for more details.
#

use strict;
use warnings;

require Exporter;

our @ISA = qw(Exporter);
our @EXPORT = qw(&gen_html_table &new);
our $VERSION = '0.01';


sub new {
    my $classname  = shift;         # What class are we constructing?
    my $self      = {};             # Allocate new memory
    bless($self, $classname);       # Mark it of the right type
    $self->_init(@_);               # Call _init with remaining args
    return $self;
} 

# "private" method to initialize fields.  It always sets START to
# the current time, and AGE to 0.  If called with arguments, _init
# interprets them as key+value pairs to initialize the object with.
sub _init {
    my $self = shift;
#$self->{tablestyle} = "border:1px solid black;"; 	# Default values
#   $self->{cellstyle} = "border:1px solid black;";
    $self->{headerrowstyle} = "background:lightgrey;";
    if (@_) {
        my %extra = @_;
        @$self{keys %extra} = values %extra;
	}
} 

sub set {
	my $self = shift;
	if (@_) {
        my %extra = @_;
        @$self{keys %extra} = values %extra;
	}
}

# Returns: html_buf
sub gen_html_table {
	my $self = shift;
	
	my ($html_results, $sth);
	my ($tablestyle, $headercellstyle, $cellstyle, $headerrowstyle, $rowstyle);
	my ($evenrowstyle, $oddrowstyle);
	my $css;
	my ($front_extension_column, $back_extension_column);
	my ($custom_header, $custom_header_template);
	# query and dbh must be defined
	die "ERROR: query must be defined." unless $self->{query};
	die "ERROR: dbh must be defined." unless $self->{dbh};
	
	$sth = $self->{dbh}->prepare( $self->{query} );

	if (not $sth->execute()) {
		$html_results = "Query: " . $self->{query} . "<br><font color=red><b>" . $sth->errstr . "</b></font><br><br>";
		return;
	}
	
	my @col_headers = @{$sth-> {NAME_uc}}; # Now you have the field names in @Header.
	
	if( defined $self->{back_extension_column} ) {
		$back_extension_column = $self->{back_extension_column};
		@col_headers=(@col_headers, $back_extension_column);
	}
	if( defined $self->{front_extension_column} ) {
		$front_extension_column = $self->{front_extension_column};
		@col_headers=($front_extension_column, @col_headers);
	}
	if( defined $self->{custom_header} ) {
		$custom_header = $self->{custom_header};
		$custom_header_template = $self->{custom_header_template};
	}
	
	my $col_num = scalar(@col_headers);
	
	# set style if available
	$tablestyle = "style=\"" . $self->{tablestyle} . "\"" 			if( defined $self->{tablestyle} );
	$headercellstyle = "style=\"" . $self->{headercellstyle} . "\"" if( defined $self->{headercellstyle} );
	$cellstyle = "style=\"" . $self->{cellstyle} . "\"" 			if( defined $self->{cellstyle} );
	$headerrowstyle = "style=\"" . $self->{headerrowstyle} . "\"" 	if( defined $self->{headerrowstyle} );
	$rowstyle = "style=\"" . $self->{rowstyle} . "\"" 				if( defined $self->{rowstyle} );
	
	if( defined $self->{twocolorrows} ) {
		$evenrowstyle = "style=\"" . $self->{evenrowstyle} . "\"";
		$oddrowstyle  = "style=\"" . $self->{oddrowstyle} . "\"";
	} else {
		$evenrowstyle = $rowstyle;
		$oddrowstyle  = $rowstyle;
	}

	if( defined $self->{rowhighlight} ) {
		
		$css = "\n<style type=\"text/css\">\n";
		$css .= "<!--\n";
		$css .= "#evenrow {\n";
		$css .= "  $self->{evenrowstyle}\n";
		$css .= "}\n";
		$css .= "#evenrow:hover {\n";
		$css .= "  $self->{evenrowstylehover}\n";
		$css .= "}\n";
		$css .= "#oddrow {\n";
		$css .= "  $self->{oddrowstyle}\n";
		$css .= "}\n";
		$css .= "#oddrow:hover {\n";
		$css .= "  $self->{oddrowstylehover}\n";
		$css .= "}\n";
		$css .= "-->\n";
		$css .= "</style>\n";
		
		$html_results = $html_results . $css;
		
		if( defined $self->{twocolorrows} ) {
			$evenrowstyle      = "id=\"evenrow\"";
			$oddrowstyle       = "id=\"oddrow\"";
		} else {
			$evenrowstyle      = "id=\"evenrow\"";
		}
	}
	
	# Custom cells, hash contains column_id as key, customize instructions as perl code (we will run eval later)
	my %expr_hash = %{$self->{expr}} if defined %{$self->{expr}};
	
	# render table
	$html_results .= "<table $tablestyle>\n";
	$html_results .= "  <tr $headerrowstyle>\n";
	foreach my $td (@col_headers) {
		
		if($td eq $back_extension_column) {
			$html_results .= "<th $headercellstyle>$back_extension_column</th>\n";
		}
		
		elsif($td eq $front_extension_column) {
			$html_results .= "<th $headercellstyle>$front_extension_column</th>\n";
		}
		else {
			if($custom_header) {
				(my $header__ = $custom_header_template) =~ s/__HEADER_TITLE__/$td/g;
				$html_results .= "  <th $headercellstyle><b><a href=\"\">$header__</a></b></th>\n";
			}
			else {
				$html_results .= "  <th $headercellstyle><b>$td</b></th>\n";
			}
		}
	}
	$html_results .= "  </tr>\n";
	
	my $cnt=0;
	while ( my @row_arr = $sth->fetchrow_array ) {
		if( defined $self->{back_extension_column} ) {
			@row_arr = (@row_arr, $back_extension_column);
		}
		if( defined $self->{front_extension_column} ) {
			@row_arr = ($front_extension_column, @row_arr);
		}
		if($cnt % 2) {
			$html_results .= "  <tr $oddrowstyle>\n";
		} else {
			$html_results .= "  <tr $evenrowstyle>\n";
		}
		
		my $colid=0;
		foreach my $td (@row_arr) {
			if($td eq $back_extension_column) {
				my $back_extension_content = $self->{back_extension_content};
				my @colidx = ($back_extension_content =~ /__COL(\d+)__/gi);      # get all matches
				foreach my $i (@colidx) {
					
					my $m = "__COL" . $i . "__";
					$i++ if(defined $front_extension_column);
					$back_extension_content =~ s/$m/@row_arr[$i]/;
				}
				$html_results .= "    <td $cellstyle>$back_extension_content</td>\n";
			} 
			elsif ($td eq $front_extension_column) {
				my $front_extension_content = $self->{front_extension_content};
				my @colidx = ($front_extension_content =~ /__COL(\d+)__/gi);      # get all matches
				foreach my $i (@colidx) {
					
					my $m = "__COL" . $i . "__";
					$i++;
					$front_extension_content =~ s/$m/@row_arr[$i]/;
				}
				$html_results .= "    <td $cellstyle>$front_extension_content</td>\n";
			}
# 			elsif ($colid == 1) {
			elsif ( $expr_hash{$colid} ne "" ) {
				my $col_content = $td;
				
				my $expr = $expr_hash{$colid};
# 				print "expr: $expr<br>";
				#my $expr = '$col_content =~ s/(.*)(\.[zy]a.exp)/<a href="?viewallreads=$1" target="__aaces_allreads_$1">-><-<\/a>$1$2/; ';
				eval $expr;
# 				$col_content =~ s/(.*)(\.[zy]a.exp)/$1$2<a href="$self->{script_loc}?viewallreads=$1" target="__aaces_allreads_$1">-><-<\/a>/;
				$html_results .= "    <td $cellstyle>$col_content</td>\n";
			}
			else {
				$html_results .= "    <td $cellstyle>$td</td>\n";
			}
			$colid++;
		}
		$html_results .= "  </tr>\n";
		$cnt++;
	}
	
	$html_results .= "</table>\n";
	$sth->finish();
	return $html_results;
}


1;
__END__
# Below is stub documentation for your module. You'd better edit it!

=head1 NAME

HTML::SQLQuery2HTMLTable - Perl extension for blah blah blah

=head1 SYNOPSIS

  use HTML::SQLQuery2HTMLTable;
  blah blah blah

=head1 DESCRIPTION

Stub documentation for HTML::SQLQuery2HTMLTable, created by h2xs. It looks like the
author of the extension was negligent enough to leave the stub
unedited.

Blah blah blah.

=head2 EXPORT

None by default.



=head1 SEE ALSO

Mention other useful documentation such as the documentation of
related modules or operating system documentation (such as man pages
in UNIX), or any relevant external documentation such as RFCs or
standards.

If you have a mailing list set up for your module, mention it here.

If you have a web site set up for your module, mention it here.

=head1 AUTHOR

A. U. Thor, E<lt>jfitz@internetlabE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2006 by A. U. Thor

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.8 or,
at your option, any later version of Perl 5 you may have available.


=cut
