------------------------------------------------------------------------
-- File: setup_msqtschema_chrpos.sql
-- Author: Joffrey Fitz
-- Usage:
--   psql -U postgres msqtpublic < ./setup_msqtschema_chrpos.sql
--
-- Clean up with:
-- echo "drop schema yourdatasetname cascade" | \
--   psql -U postgres msqtpublic
-- echo "delete from msqtpublic.public.msqt_config where msqt_schema_name='YourDatasetName'" | \
--   psql -U postgres msqtpublic
------------------------------------------------------------------------

BEGIN;
------------------------------------------------------------------------
-- Create new schema
------------------------------------------------------------------------
create schema YourDatasetName;

------------------------------------------------------------------------
-- Create table structure in the new schema of type 
-- struct_position_chromosome.
--
-- We need 5 tables: 
-- 	ecotypes
--	snps
--	target_sequences
--	primers
--	frag_id2assay
------------------------------------------------------------------------

CREATE TABLE YourDatasetName.ecotypes 
(
	ecotype varchar(8)
);

CREATE TABLE YourDatasetName.snps (
	chromosome int4 NOT NULL DEFAULT 0,
	"position" int4 NOT NULL DEFAULT 0,
	ecotype varchar(8) NOT NULL DEFAULT ''::character varying,
	replacement char(1),
	frag_id int4,
	allele_freq float4 DEFAULT 1,
	CONSTRAINT YourDatasetName_snps_pkey PRIMARY KEY ("position", chromosome, ecotype)
);

CREATE TABLE YourDatasetName.target_sequences
(
	chromosome int4 NOT NULL DEFAULT 0,
	"position" int4 NOT NULL DEFAULT 0,
	ref_base char(1),
	frag_id int4,
	CONSTRAINT YourDatasetName_target_sequences_pkey PRIMARY KEY ("position", chromosome)
);

CREATE TABLE YourDatasetName.primers
(
	chromosome int4 NOT NULL,
	frag_id int4 NOT NULL,
	"position" int4,
	query_name varchar(15),
	genaissance_assay_id int4 NOT NULL DEFAULT 0,
	gnsc_snp_id int4,
	second_pcrp varchar(40),
	first_pcrp varchar(40),
	probe_direction char(3),
	probe varchar(40),
	CONSTRAINT YourDatasetName_primers_pkey PRIMARY KEY (genaissance_assay_id)
);

CREATE TABLE YourDatasetName.frag_id2assay
(
	genaissance_assay_id int4 NOT NULL DEFAULT 0,
	chromosome int4 NOT NULL,
	frag_id int4 NOT NULL,
	CONSTRAINT YourDatasetName_frag_id2assay_pkey PRIMARY KEY (genaissance_assay_id)
);

CREATE TABLE YourDatasetName.sniped
(
  chrom int2 NOT NULL,
  fragid int4 NOT NULL,
  "position" int4 NOT NULL,
  summary text,
  allele1 varchar(1) NOT NULL,
  allele_freq1 float4 NOT NULL,
  allele2 varchar(1) NOT NULL,
  allele_freq2 float4 NOT NULL,
  freq_sum float4 NOT NULL,
  freq_diff float4 NOT NULL,
  neighborhood_left_len int4 NOT NULL,
  neighborhood_right_len int4 NOT NULL,
  ecot1 text NOT NULL,
  ecot2 text NOT NULL,
  adf text NOT NULL
);

------------------------------------------------------------------------
-- Set user privileges for msqtwebuser
------------------------------------------------------------------------
grant usage on SCHEMA YourDatasetName to msqtwebuser;
grant select on TABLE YourDatasetName.snps to msqtwebuser;
grant select on TABLE YourDatasetName.ecotypes to msqtwebuser;
grant select on TABLE YourDatasetName.primers to msqtwebuser;
grant select on TABLE YourDatasetName.target_sequences to msqtwebuser;
grant select on TABLE YourDatasetName.frag_id2assay to msqtwebuser;
grant select on TABLE YourDatasetName.sniped to msqtwebuser;

------------------------------------------------------------------------
-- Register type of schema in msqt_config
--
-- Required information: 
--   name of the schema:    arbitrary text
--   type of the schema:    struct_position_name 
--                          or struct_position_chromosome
--   number of chromosomes: only required if type is 
--                          struct_position_chromosome
------------------------------------------------------------------------

-- Example for type struct_position_chromosome (e.g. Nordborg data)
INSERT INTO public.msqt_config 
(
	msqt_schema_name,
	msqt_schema_type,
	chromosomes,
	uploaded_archive,
	population_date
)
VALUES (
	'YourDatasetName',
	'struct_position_chromosome',
	YourNumberOfChromosomes,
	'YourArchiveFile',
	'PopulationDate'
);


COMMIT;
