#!/usr/bin/perl
use Getopt::Std;

# Joff data processor
# Copyright (c) 2006 by Joffrey Fitz
#
# usage:
#  ./jdproc.pl < mysql-gentoo.txt > mysql-gentoo.html
#
# text format tags:
#
# headline=
# author=
# date=
# email=
# h1=
# code=.*=
# - (list item)

use strict;

my $version = "2006.1 (0.4)";

my $code = 0;
my $seccnt=0;
my $h2=0;
my $h3=0;
my ($document_headline, $document_author, $document_date, $document_email);
my ($htmlheader, $style, $header, $authorline, $contents, $body, $copyright, $htmlfooter);
my $buffer;

# Parse command line options
my %option = ();
# options for header file (-h), footer file (-f) and stylesheet (-s)
# option to ommit copyright line (-c)
# option to ommit author credits (-a)
getopts("h:f:s:S:ca", \%option);

if ($option{'s'}) {
    open(FIN, "< $option{'s'}" ) 
        || die "Can't open $option{'s'}. Exiting.\n";
	while(<FIN>) {
		$style .= $_;
	}
	close(FIN);
}
elsif($option{'S'}) {
	# do not process any css
}
else {
$style = <<EOS;

body { background: #F4F4F4 none;
  padding-top: 0em;
  font-family:verdana,helvetica; font-size:12px; color:black;}



h1 { 
}

h2 { }

h3 { }

hr { color: #436EEE;
     background-color: #436EEE;
     border: 0;
     height: 3px;
     padding-left: 1em;
}

h1,h2,h3,h4,p,ul,ol,li,div,td,th,address,blockquote,nobr,b,i {
	font-family:verdana,helvetica; 
}
p,ul,ol,li,div,td,th,address,blockquote { 
	font-size:13.4px; 
}

/* four different title sections */
.jdchapter { 
	font-family:verdana,helvetica;
	font-size:21px; 
	margin-bottom:8px; 
	margin-top:28px; 
	font-weight:bold; 
	color: #436EEE; 
}
.jdsection { 
	color:#4e4e4e; 
	font-family:verdana,helvetica; 
	font-size:21px; 
}
.jdsubsection { 
	font-family:verdana,helvetica;
	font-size:16px; 
	margin-bottom:8px; 
	margin-top:28px; 
	font-weight:bold; 
	color: #436EEE;
}
.jdsubsubsection { 
	font-family:verdana,helvetica;
	font-size:12px; 
	margin-bottom:8px; 
	margin-top:18px; 
	font-weight:bold; 
	color: black;
}

.text { font-family:verdana,helvetica; font-size:10pt; line-height:12pt; color:black;}
.tinycomment { font-family:verdana,helvetica; font-style:normal; font-size:9pt; line-height:9pt; color:black;}


#code {
  border-width:1px;
  border-style:solid;
  //border-top-color:#86b7e7;
  //border-left-color:#86b7e7;
  //border-right-color:#809fbd;
  //border-bottom-color:#809fbd;
  border-color: #e0e0e0;
  background-color:#e8e8e8;
  padding:5px;
  text-align:justify;
  font-size:12px;
  font-family:'Courier New',Courier; 
}
#text {
	font-family:sans-serif;
	font-size:10pt;
}
#textcontainer {
  padding-left: 0.5em;
  padding-right: 0.5em;
}
pre { 
	font-family:'Courier New',Courier; 
	font-size:13.4px; 
}
#copy {
  font-size:9px;
}
EOS
}

# Header
$htmlheader = <<EOH;
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Strict//EN"
       "http://www.w3.org/TR/html4/strict.dtd">
<html>
<head>
<title>--Title--</title>
<meta name="author" content="--Author--">
<style type="text/css">
<!--
$style
-->
</style>
</head>
EOH

if ($option{'h'}) {
    open(FIN, "< $option{'h'}" ) 
        || die "Can't open $option{'h'}. Exiting.\n";
	while(<FIN>) {
		$htmlheader .= $_;
	}
	close(FIN);
}
else {
$htmlheader .= <<EOH;
<body>

EOH
}

# Print copyright ?
if (not $option{'c'}) {
$copyright = <<EOC;
<p><br></div><hr>
<div id="copy">This document was generated on --Date-- using jdproc Version $version written
by Joffrey Fitz</div>
EOC
}

# authorline?
if (not $option{'a'}) {
$authorline = <<EOC;
</div><hr><i>--Date-- by --Author-- (--Email--)</i>
EOC
}

# Footer
if ($option{'f'}) {
    open(FIN, "< $option{'f'}" ) 
        || die "Can't open $option{'f'}. Exiting.\n";
	while(<FIN>) {
		$htmlfooter .= $_;
	}
	close(FIN);
}
else {
$htmlfooter = <<EOF;
</td></tr></table>
</div>
</body>
</html>
EOF
}


# init content
$contents =<<EOT;
<div class="doc_container">
<table border=0 cellpadding=0 cellspacing=0>
<tr><td width="520" align=justify>
<h2 class="jdsubsection">Contents</h2>
<ul>
EOT

# write stuff
while(<>) {
	if( /^headline=(.*)/ ) {
		my $title = $1;
		$document_headline .= "<h1 class=\"jdsection\">$title</h1>\n";
		$htmlheader =~ s/--Title--/$title/;
		next;
	}
	elsif( /^author=(.*)/ ) {
		my $author = $1;
		$document_author= "$1";
		$htmlheader =~ s/--Author--/$author/;
		next;
	}
	elsif( /^date=(.*)/ ) {
		my $date = $1;
		$document_date = $1;
		$htmlheader =~ s/--Date--/$date/;
		next;
	}
	elsif( /^email=(.*)/ ) {
		my $author = $1;
		$document_email = $1;
		next;
	}
	elsif( /^h1=(.*)/ ) {
		$body .= "<h1 class=\"jdsection\">$1</h1>\n";
		next;
	}
	elsif( /^chap=(.*)/ ) {
		if($h2) { $h2=0; $contents .= "</ul>"; }
		if($h3) { $h3=0; $contents .= "</ul>"; }
		$contents .= "<li><a href=\"#Sec$seccnt\">$1</a></li>\n";
		$body .= "<h2 class=\"jdchapter\"><a name=\"Sec$seccnt\">$1</a></h2>\n";
		
		$seccnt++;
		next;
	}
	elsif( /^h2=(.*)/ ) { #print STDERR "in h2\n";
		if($h3) { $h3=0; $contents .= "</ul>"; }
		if(!$h2) { 
			$h2=1; 
			$contents .= "<ul><li><a href=\"#Sec$seccnt\">$1</a></li>\n";
			$body .= "<p><a href=\"#Headline\">[Up]</a><br><br><p>";
			$body .= "<h2 class=\"jdsubsection\"><a name=\"Sec$seccnt\">$1</a></h2>\n";
		}
		else {
			$contents .= "<li><a href=\"#Sec$seccnt\">$1</a></li>\n";
			$body .= "<p><a href=\"#Headline\">[Up]</a><br><br><p>";
			$body .= "<h2 class=\"jdsubsection\"><a name=\"Sec$seccnt\">$1</a></h2>\n";
			$seccnt++;
		}
		next;
	}
	elsif( /^h3=(.*)/ ) { #print STDERR "  in h3\n";
		if(!$h3) { 
			$h3=1; 
			$contents .= "<ul><li><a href=\"#Sec$seccnt\">$1</a></li>\n"; 
			$body .= "<h3 class=\"jdsubsubsection\"><a name=\"Sec$seccnt\">$1</a></h3>\n";
		}
		else {
			$contents .= "<li><a href=\"#Sec$seccnt\">$1</a></li>\n";
			$body .= "<h3 class=\"jdsubsubsection\"><a name=\"Sec$seccnt\">$1</a></h3>\n";
			$seccnt++;
		}
		next;
	}
	# code
	elsif( /^code=(.*)/ ) {
		$code=1;
		$body .= "<pre><p id=\"code\">$1<br>";
		next;
	}
	elsif( /^=/ ) {
		if( $code ) { $body .= "</pre>"; $code=0; }
		$body .= "</p>\n";
		next;
	}

	# list
	elsif( /^- (.*)/ ) {
		$body .= "<li id=\"jdproclist\">$1</li>\n";
		next;
	}
	# new paragraph
        elsif( /^-p$/ ) {
		$body .= "<p>\n";
		next;
	}
	else {
		if( $code ) {
			$body .= "$_";
		}
		else {
			$body .= "$_\n";
		}
	}
}


# terminate contents
$contents .= "</ul></ul>\n";

# insert date into copyright line
$copyright =~ s/--Date--/$document_date/;

# compile author line
$authorline =~ s/--Date--/$document_date/;
$authorline =~ s/--Author--/$document_author/;
$authorline =~ s/--Email--/$document_email/;

$buffer = $htmlheader.$header.$document_headline.
	$authorline. 
	$contents.$body.
	"<p><a href=\"#Headline\">[Up]</a></p>".
	$copyright.$htmlfooter;
print $buffer;
