package distinct_snps_posname;

#
# distinct_snps_posname.pm
#
# Authors: 
#	Joffrey Fitz <fitzj@informatik.uni-tuebingen.de>
#   Norman Warthmann <norman.warthmann@tuebingen.mpg.de>
#   Stephan Ossowski <stephan.ossowski@tuebingen.mpg.de>
#
# Copyright (c) 2007 Max Planck Institute for Developmental Biology, 
# 	http://www.eb.tuebingen.mpg.de
#
# This program is part of the MSQT project by Norman Warthmann, Joffrey Fitz 
# and Detlef Weigel (http://msqt.weigelworld.org)
#
# This program is free software; you can redistribute it and/or modify it                        
# under the terms of the GNU General Public License as published by the Free                     
# Software Foundation; either version 2, or (at your option) any later 
# version.
#
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
# or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
# for more details.
#

# Attention:
# !!! This is a branch for arabis_alpina and guppy !!!

use strict;
use warnings;
use DBI;
use Switch;


sub new {
	my $self = {
		chromosome => '',
		start => '',
		end => '',
		dbtable => '',
		dbh => '',
		gen_out => '',
		ecotypes => {},
		distinct_snp_hash => {},
		all_snp_hash => {},
		target_snp_hash => {},
		all_frag_hash => {},
		output => [],
	};
	bless $self;
	return $self;
}

# modified init for arabis_alpina, guppy
sub init {
	#my ($self, $chromosome, $start, $end, $dbh, $dbtable, %ecotypes) = @_;
	#$self->{chromosome} = $chromosome;
	#$self->{start} = $start;
	#$self->{end} = $end;
	#%{$self->{ecotypes}} = %ecotypes;
	#$self->{dbtable} = $dbtable;

  	my ($self, $chromosome, $frag_id, $pos_name, $dbh, $dbtable, %ecotypes) = @_;
	%{$self->{ecotypes}} = %ecotypes;
	$self->{dbtable} = $dbtable;

	my $ecotype_list_one = '';
	my $ecotype_list_two = '';
	my $ecotype_list_three = '';
	my %ecotype_hash_one = ();
	my %ecotype_hash_two = ();
	my $current_position = -1;
	my @A = (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0);
	my @B = (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0);
	my @C = (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0);
	my @D = (0,0,0,0,0);
	my $buf = '';

	# Prepare two lists of ecotypes for SQL statement
	foreach my $ecotype (keys %ecotypes){
		if($ecotypes{$ecotype} == 0) {
			$ecotype_list_one = $ecotype_list_one . " (ecotype='$ecotype') ";
			$ecotype_hash_one{$ecotype} = 1;
		}
		elsif($ecotypes{$ecotype} == 1) {
			$ecotype_list_two = $ecotype_list_two . " (ecotype='$ecotype') ";
			$ecotype_hash_two{$ecotype} = 1;
		}
		elsif($ecotypes{$ecotype} == 2) {
			$ecotype_list_three = $ecotype_list_three . " (ecotype='$ecotype') ";
			#$ecotype_hash_two{$ecotype} = 1;
		}
	}
	$ecotype_list_one=~s/\)  \(/\) or \(/g;
	$ecotype_list_two=~s/\)  \(/\) or \(/g;
	$ecotype_list_three=~s/\)  \(/\) or \(/g;

	# we dont have $end and $start is position_name
	my $frag_id_silencer='';
	
	if($frag_id eq "") {
		$frag_id_silencer = "--";
	}
	
	my $pos_name_silencer='';
	if($pos_name eq "") {
		$pos_name_silencer = "--";
	}
	
#print "DEBUG:<pre>silencer:\nfrag_id:$frag_id, frag_id_silencer:$frag_id_silencer\n",
#		"pos_name:$pos_name, pos_name_silencer:$pos_name_silencer</pre>";
	
	# Select all snp for range and ecotypes
# 	my $q = 	"SELECT position_name, ecotype, replacement, frag_id
# 			 FROM 	$dbtable
# 			 WHERE
#                           --chromosome = $chromosome AND
# 			  --$silencer position BETWEEN $start AND $end AND
# 				($ecotype_list_one OR $ecotype_list_two or (ecotype='target'))
# 			 ORDER BY split_part
#                           ( position_name, '-', 1)::text, split_part( position_name, '-', 2)::int;
# 			";

	my $q = <<EOQ;
SELECT position_name, ecotype, replacement, frag_id
	FROM $dbtable.snps
    	WHERE
			$frag_id_silencer frag_id LIKE '$frag_id' AND
            $pos_name_silencer position_name LIKE '$pos_name' AND
            ($ecotype_list_one OR	-- ecotype_list_one
				$ecotype_list_two OR	-- ecotype_list_two
				$ecotype_list_three)	-- ecotype_list_three
     	ORDER BY split_part
        	( position_name, '-', 1)::text, split_part( position_name, '-', 2)::int;

EOQ

#print "DEBUG:<pre>distinct_snps_posname.pm:init: q=$q</pre>";
	
	my $q_sth = $dbh->prepare($q);
	$q_sth->execute();
        #print "DEBUG q_sth executed <br>";

	# Loop through results
	my $frag_id;
	while(my $q_ref = $q_sth->fetchrow_hashref()) {
	       #print "DEBUG: Loop through results: position_name=" . $q_ref->{'position_name'} ."<br>";

		my $position = $q_ref->{'position_name'};
		my $ecotype = $q_ref->{'ecotype'};
		my $replacement = $q_ref->{'replacement'};
		$frag_id = $q_ref->{'frag_id'};

		# store frag_id
		$self->{all_frag_hash}{$position} = $frag_id;
                # Only if postion changed, process something
		if($position ne $current_position) {
                #print "DEBUG: if(position ne current_position): position_name changed: $current_position => $position<br>";

			if($current_position ne "-1") {
			#print "DEBUG: if(current_position != -1) entered<br>";

				my $A_as_number = $A[0]*1+$A[1]*10+$A[2]*100+$A[3]*1000+$A[4]*10000;
				my $B_as_number = $B[0]*1+$B[1]*10+$B[2]*100+$B[3]*1000+$B[4]*10000;
				my $C_as_number = $C[0]+$C[1]+$C[2]+$C[3]+$C[4];

				if($C_as_number > 1) {
					$self->{all_snp_hash}{$current_position} = &bv2_bases(@C);
				}
				elsif(	($C[0] != $D[0]) || ($C[1] != $D[1]) || ($C[2] != $D[2]) ||
					($C[3] != $D[3]) || ($C[4] != $D[4]) ) {
					$self->{target_snp_hash}{$current_position} = &bv2_bases(@C);
				}
				if(!(
					($A_as_number==0) ||
					($B_as_number==0) ||
					($A_as_number - $B_as_number == 0) ||
					($A[0]==1 && $B[0]==1) ||
					($A[1]==1 && $B[1]==1) ||
					($A[2]==1 && $B[2]==1) ||
					($A[3]==1 && $B[3]==1) ||
					($A[4]==1 && $B[4]==1)
				)) {
					$buf .= "$current_position\t@A\t@B\t";
					$buf .= &bv2_bases(@A); $buf .= "\t";
					$buf .= &bv2_bases(@B); $buf .= "\n";
					$self->{distinct_snp_hash}{$current_position} = &bv2_bases(@A) . "/" . &bv2_bases(@B);
				        #print "DEBUG: loop thru res: distinct_snp_hash: "
				        #  .$self->{distinct_snp_hash}{$current_position} . ", current_position: $current_position<br>";
				}
			}

			$current_position = $position;
#print "current position set: $current_position<br>";
			@A = (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0);
			@B = (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0);
			@C = (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0);
			@D = (0,0,0,0,0);
		}

		#check if different from target
		if($ecotype eq "target") {
			switch ($replacement) {
				case "A"        {$D[0]=1}
				case "T"        {$D[1]=1}
				case "G"        {$D[2]=1}
				case "C"        {$D[3]=1}
				case "-"        {$D[4]=1}
			}
		}

		#binary coding of replacement for all hash
		else{
			switch ($replacement) {
				case "A"        {$C[0]=1}
				case "T"        {$C[1]=1}
				case "G"        {$C[2]=1}
				case "C"        {$C[3]=1}
				case "-"        {$C[4]=1}
				case "N"        {$C[5]=1}
				case "R" 		{$C[6]=1}
				case "Y" 		{$C[7]=1}
				case "K" 		{$C[8]=1}
				case "M" 		{$C[9]=1}
				case "S" 		{$C[10]=1}
				case "W" 		{$C[11]=1}
				case "B" 		{$C[12]=1}
				case "D" 		{$C[13]=1}
				case "H" 		{$C[14]=1}
				case "V" 		{$C[15]=1}
				else {print "First ecotype group has problem at\t$position\n"}
			}
		}

		if(exists $ecotype_hash_one{$ecotype}) {
			#binary coding of replacements for distinct snps
			switch ($replacement) {
				case "A" 	{$A[0]=1}
				case "T" 	{$A[1]=1}
				case "G" 	{$A[2]=1}
				case "C" 	{$A[3]=1}
				case "-" 	{$A[4]=1}
				case "N" 	{$A[5]=1}
				case "R" 	{$A[6]=1}
				case "Y" 	{$A[7]=1}
				case "K" 	{$A[8]=1}
				case "M" 	{$A[9]=1}
				case "S" 	{$A[10]=1}
				case "W" 	{$A[11]=1}
				case "B" 	{$A[12]=1}
				case "D" 	{$A[13]=1}
				case "H" 	{$A[14]=1}
				case "V" 	{$A[15]=1}
				else {print "First ecotype group has problem at\t$position\n"}
			}
		}
		elsif(exists $ecotype_hash_two{$ecotype}) {
			#binary coding of replacements for distinct snps
			switch ($replacement) {
				case "A"        {$B[0]=1}
				case "T"        {$B[1]=1}
				case "G"        {$B[2]=1}
				case "C"        {$B[3]=1}
				case "-"        {$B[4]=1}
				case "N"        {$B[5]=1}
				case "R" 		{$B[6]=1}
				case "Y" 		{$B[7]=1}
				case "K" 		{$B[8]=1}
				case "M" 		{$B[9]=1}
				case "S" 		{$B[10]=1}
				case "W" 		{$B[11]=1}
				case "B" 		{$B[12]=1}
				case "D" 		{$B[13]=1}
				case "H" 		{$B[14]=1}
				case "V" 		{$B[15]=1}
				else {print "Second ecotype group has problem at\t$position\n"}
			}
		}
	}

	# Postprocessing last entry from db
	my $A_as_number = $A[0]*1+$A[1]*10+$A[2]*100+$A[3]*1000+$A[4]*10000;
	my $B_as_number = $B[0]*1+$B[1]*10+$B[2]*100+$B[3]*1000+$B[4]*10000;
	my $C_as_number = $C[0]+$C[1]+$C[2]+$C[3]+$C[4];

	if($C_as_number > 1) {
		$self->{all_snp_hash}{$current_position} = &bv2_bases(@C);
		$self->{all_frag_hash}{$current_position} = $frag_id;
	}
	elsif(  ($C[0] != $D[0]) || ($C[1] != $D[1]) || ($C[2] != $D[2]) ||
		($C[3] != $D[3]) || ($C[4] != $D[4]) ) {
		$self->{target_snp_hash}{$current_position} = &bv2_bases(@C);
	}
	if(!(
		($A_as_number==0) || ($B_as_number==0) || ($A_as_number - $B_as_number == 0) ||
		($A[0]==1 && $B[0]==1) || ($A[1]==1 && $B[1]==1) || ($A[2]==1 && $B[2]==1) ||
		($A[3]==1 && $B[3]==1) || ($A[4]==1 && $B[4]==1)
	)) {
		$buf .= "$current_position\t@A\t@B\t"; $buf .= &bv2_bases(@A);
		$buf .= "\t"; $buf .= &bv2_bases(@B); $buf .= "\n";
		$self->{distinct_snp_hash}{$current_position} = &bv2_bases(@A) . "/" . &bv2_bases(@B);
	        #print "DEBUG: Postprocessing last entry from db: distinct_snp_hash: "
		 # .$self->{distinct_snp_hash}{$current_position} . ", current_position: $current_position<br>";
	}

	# Create clustered output array
	$self->cluster($chromosome, $buf, $dbtable);
}

# modified cluster for arabis_alpina, guppy
sub cluster {
	my ($self, $chromosome, $buf, $dbtable) = @_;

	my @positionsarray = ();
	my @seq1array = ();
	my @seq2array = ();
	my $start;
        my @rel_positions = (); # new
        my @tmp = ();#new
	my @lines = split("\n",$buf);

	foreach my $line (@lines) {
		chomp($line);
		my ($position, $bin1, $bin2, $base1, $base2) = split("\t",$line);
                push(@positionsarray, $position);
                push(@seq1array,"$base1");
		push(@seq2array,"$base2");

                # New
                @tmp=split(/-/, $position); # new
                push(@rel_positions, $tmp[1]);
	}

	my $flag=0;
	my @indel_allele1=();
	my @indel_allele2=();
	my $tmp = '';
	my $xmlout = '';

	push(@positionsarray, 0);
	
	$xmlout = "<?xml version=\"1.0\"?>\n";
	$xmlout .= "<msqtsbe>\n";
	push(@{$self->{output}}, $xmlout);
	for (my $i=1; $i < scalar(@positionsarray); $i++) {
		if (($flag==0) && ($rel_positions[$i] != $rel_positions[$i-1]+1)) {
			
			$xmlout = "\t<polymorphism>\n";
			$xmlout .=  "\t\t<type>SNP</type>\n";
			#$xmlout .=  "\t\t<chromosome>$chromosome</chromosome>\n";
			$xmlout .= "\t\t<fragid>".$self->{all_frag_hash}{$positionsarray[$i-1]}."</fragid>\n";
			$xmlout .= "\t\t<position>$positionsarray[$i-1]</position>\n";
			$xmlout .= "\t\t<allele1>$seq1array[$i-1]</allele1>\n";
			$xmlout .= "\t\t<allele2>$seq2array[$i-1]</allele2>\n";
			
			$xmlout .= "\t</polymorphism>\n";
			push(@{$self->{output}}, $xmlout);

		}
		elsif (($flag==0) && ( $rel_positions[$i] == $rel_positions[$i-1]+1)) {
			$flag=1;
			
			$xmlout = "\t<polymorphism>\n";
			$xmlout .= "\t\t<type>indel</type>\n";
			$xmlout .= "\t\t<chromosome>$chromosome</chromosome>\n";
			$xmlout .= "\t\t<position>$positionsarray[$i-1]</position>\n";

			push(@indel_allele1,"$seq1array[$i-1]");
			push(@indel_allele2,"$seq2array[$i-1]");
		}
		elsif (($flag==1) && ($rel_positions[$i] == $rel_positions[$i-1]+1)) {
			push(@indel_allele1,"$seq1array[$i-1]");
			push(@indel_allele2,"$seq2array[$i-1]");
		}
		elsif (($flag==1) && ($rel_positions[$i] != $rel_positions[$i-1]+1)) {
			$flag=0;
			push(@indel_allele1,"$seq1array[$i-1]");
			push(@indel_allele2,"$seq2array[$i-1]");
			
			$xmlout .= "\t\t<fragid>".$self->{all_frag_hash}{$positionsarray[$i-1]}."</fragid>\n";
			$xmlout .= "\t\t<allele1>" . join ("",@indel_allele1) . "</allele1>\n";
			$xmlout .= "\t\t<allele2>" . join ('',@indel_allele2) . "</allele2>\n";				
			$xmlout .= "\t</polymorphism>\n";
				
			push(@{$self->{output}}, $xmlout);
			@indel_allele1=();
			@indel_allele2=();
		}
		else { print  $chromosome ."\tthere is somthing wrong!\n"; }
	}
	$xmlout = "</msqtsbe>\n";
	push(@{$self->{output}}, $xmlout);
}


#####################################################
### backtranslates the binary code into bases
#####################################################
sub bv2_bases
{
	my @bases = ('A','T','G','C','-','N','R','Y','K','M','S','W','B','D','H','V');
	my $buf;
	my @bv = @_;

	# Uebersetze Bitvector in Basen
	for(my $i=0; $i<scalar(@bv); $i++) {
		$buf.= "$bases[$i]," if $bv[$i];
	}
	chop($buf);
	return $buf;
}

# modified find_frag for arabis_alpina, guppy
# we don't use this coz we know the frag
sub find_frag {
    # Get fragment of distinct snp
	#
	#
	# !!! If pos_name is given, only the primary SNP will be returned !!!
	#
	#
	my ($self, $chromosome, $frag_id, $pos_name, $dbh, $dbtable, %ecotypes) = @_;
	#print "DEBUG: distinct_snps.pm: find_frag() called with: ";
	#print "DEBUG: find_frag: chr=$chromosome, position=$position, dbh, dbtable=$dbtable, ...<br>";

	
# 	$q_frag =    	"SELECT min(position) AS minpos, max(position) AS maxpos
# 			FROM 	target_sequence_table
# 			WHERE 	frag_id = $frag_id
# 			";
# 	$sth_frag = $dbh->prepare($q_frag);
# 	$sth_frag->execute();
# 	$ref_frag = $sth_frag->fetchrow_hashref();
# 	my $start = $ref_frag->{'minpos'};
# 	my $end   = $ref_frag->{'maxpos'};
	$self->init($chromosome, $frag_id, $pos_name, $dbh, $dbtable, %ecotypes);
}

sub genaissance
{
	my ($self, $dbh) = @_;

        # Get target sequence of fragment
	my $q_target = "SELECT position, ref_base FROM target_sequence_table
			WHERE   chromosome = " . $self->{chromosome} . " AND
			position BETWEEN " . $self->{start} . " AND " . $self->{end} . "
			ORDER BY position
			";
	my $sth_target = $dbh->prepare($q_target);
	$sth_target->execute();
	while(my $ref_target = $sth_target->fetchrow_hashref()) {
		my $position = $ref_target->{'position'};
		my $base = $ref_target->{'ref_base'};
		if(exists $self->{distinct_snp_hash}{$position}) {
			$self->{gen_out} = $self->{gen_out} . "[" . $self->{distinct_snp_hash}{$position} . "]";
}
		elsif(exists $self->{all_snp_hash}{$position}) {
			$self->{gen_out} = $self->{gen_out} . "{" . $self->{all_snp_hash}{$position} . "}";
}
		elsif(exists $self->{target_snp_hash}{$position}) {
			$self->{gen_out} = $self->{gen_out} . "{" . $self->{target_snp_hash}{$position} . "}";
}
		else{
			$self->{gen_out} .= $base;
}
}
}


sub genaissance_html
{
        #print "DEBUG: genaissance_html() entered<br> ";
	my ($self, $dbh2, $frag_id) = @_;
	my $primary_snp_pos;

	# Get target sequence of fragment
	my $q_target = "SELECT split_part( position_name, '-', 2)::int AS position_name, ref_base FROM " . $self->{dbtable} . ".target_sequences
			WHERE
			frag_id = '$frag_id'
			ORDER BY split_part
                        ( position_name, '-', 1)::text, split_part( position_name, '-', 2)::int;
			";


        #print "DEBUG: distinct_snps.pm: genaissance_html(): <pre>q_target: $q_target</pre>";

	my $sth_target = $dbh2->prepare($q_target) || die $dbh2->errstr;
	$sth_target->execute()  || die "$!";

	my @pos_arr;
	my %target;
	while(my $ref_target = $sth_target->fetchrow_hashref()) {
		$target{ $ref_target->{'position_name'} } = $ref_target->{'ref_base'};
	}

	my $i=0;

	#DEBUG
	#print "DEBUG distinct_snp_hash:<br>";
	#foreach my $key (sort keys %{$self->{distinct_snp_hash}} ) {
	# print " $key<br>";
	#}
	#print "DEBUG: /distinct_snp_hash<br>";
	#print "DEBUG target:<br>";
	#foreach my $key (sort {$a <=> $b} keys %target ) {
	# print " $key<br>";
	#}
	#print "DEBUG: /distinct_snp_hash<br>";
	#/DEBUG
	foreach my $position (sort {$a <=> $b} keys %target) {
	        #my $value = $target{$position};
	        my $position_name = $frag_id."-$position";
	        #print "DEBUG: genaissance_html: foreach#1: position_name: $position_name<br>";

		if(exists $self->{distinct_snp_hash}{$position_name}) {
		        #print "found snp at $position_name<br>";

		        $i++;
			$self->{gen_out} = $self->{gen_out} . "<$i>:";
			#print "DEBUG i=$i<br>";
			foreach my $position2 (sort {$a <=> $b} keys %target) {
                                my $position_name2 = $frag_id."-$position2";
                                #print "DEBUG: genaissance_html: foreach#1: position_name: $position_name<br>";

				my $value = $target{$position2};

				if($position2 == $position) { # this is the primary snp
					$primary_snp_pos = $position;
					$self->{gen_out} = $self->{gen_out} . "[" . $self->{distinct_snp_hash}{$position_name2} . "]";
				}
				elsif(exists $self->{all_snp_hash}{$position_name2}) {
					$self->{gen_out} = $self->{gen_out} . "{" . $self->{all_snp_hash}{$position_name2} . "}";
				}
				elsif(exists $self->{target_snp_hash}{$position_name2}) {
					$self->{gen_out} = $self->{gen_out} . "{" . $self->{target_snp_hash}{$position_name2} . "}";
				}
				else{
					$self->{gen_out} .= $value;
				}
			}
			#$self->{gen_out} =~ s/<$i>/$primary_snp_pos/;
			$self->{gen_out} =~ s/<$i>/$position_name/;
			$self->{gen_out} .= "\n\n";#"<br><br>";
		}
	}
	# delete dashes outside of snps
	$self->{gen_out} =~ s/([A-Z])-+?([A-Z])/$1$2/g;
	# highlight
	$self->{gen_out} =~ s/(\[.+?\])/<font color="red"><b>$1<\/b><\/font>/g;
	$self->{gen_out} =~ s/(\{.+?\})/<font color="blue"><b>$1<\/b><\/font>/g;
}

sub genaissance_xml_experimental
{
	my ($self, $dbh2, $pos_name) = @_;
	my $primary_snp_pos;
	my $xml_out;
	
	my ($frag_id, $position) = split(/-/, $pos_name);
	
	# Get target sequence of fragment
	my $q_target = "SELECT split_part( position_name, '-', 2)::int AS position_name, ref_base FROM " . $self->{dbtable} . ".target_sequences
			WHERE
			frag_id = '$frag_id'
			ORDER BY split_part
                        ( position_name, '-', 1)::text, split_part( position_name, '-', 2)::int;
			";

	my $sth_target = $dbh2->prepare($q_target) || die $dbh2->errstr;
	$sth_target->execute()  || die "$!";

	my @pos_arr;
	my %target;
	while(my $ref_target = $sth_target->fetchrow_hashref()) {
		$target{ $ref_target->{'position_name'} } = $ref_target->{'ref_base'};
	}

	my $i=0;

	my $position_name = $frag_id."-$position";

	if(exists $self->{distinct_snp_hash}{$position_name}) {
		
		# Start XML
		$xml_out .= "<msqtadf structure=\"posname\">\n<fragment>\n";
		$xml_out .= "\t<frag_id>" . $frag_id . "</frag_id>\n";
		$xml_out .= "\t<position_name>" . $position_name . "</position_name>\n";
		
		# Assemble ecotype list
		my ($et1, $et2, $et3);
		foreach my $key (sort keys %{$self->{ecotypes}} ) {
			my $value = ${$self->{ecotypes}}{$key};
			if($value eq "0") { $et1 .= "\t\t<ecotype>$key</ecotype>\n"; }
			if($value eq "1") { $et2 .= "\t\t<ecotype>$key</ecotype>\n"; }
			if($value eq "2") { $et3 .= "\t\t<ecotype>$key</ecotype>\n"; }
		}
		$xml_out .= "\t<ecotype_list1>\n$et1\t</ecotype_list1>\n";
		$xml_out .= "\t<ecotype_list2>\n$et2\t</ecotype_list2>\n";
		$xml_out .= "\t<ecotype_list3>\n$et3\t</ecotype_list3>\n";
		$xml_out .= "\t<polymorphisms>\n";
		
		$i++;
		
		foreach my $position2 (sort {$a <=> $b} keys %target) {
			my $position_name2 = $frag_id."-$position2";
			#print "DEBUG: genaissance_html: foreach#1: position_name: $position_name<br>";

			my $value = $target{$position2};
			if($position2 == $position) { # this is the primary snp
				$primary_snp_pos = $position;
				$self->{gen_out} = $self->{gen_out} . "[" . $self->{distinct_snp_hash}{$position_name2} . "]";
				my ($allele1, $allele2) = split(/\//, $self->{distinct_snp_hash}{$position_name2} );
				
				$xml_out .= "\t\t<primary_snp>\n";
				$xml_out .= "\t\t\t<position>" . $primary_snp_pos . "</position>\n";
				$xml_out .= "\t\t\t<allele1>" . $allele1 . "</allele1>\n";
				$xml_out .= "\t\t\t<allele2>" . $allele2 . "</allele2>\n";
				$xml_out .= "\t\t</primary_snp>\n";
			}
			elsif(exists $self->{all_snp_hash}{$position_name2}) {
				$self->{gen_out} = $self->{gen_out} . "{" . $self->{all_snp_hash}{$position_name2} . "}";
				my ($allele1, $allele2) = split(/,/, $self->{all_snp_hash}{$position_name2} );
			
				$xml_out .= "\t\t<secondary_snp>\n";
				$xml_out .= "\t\t\t<position>" . $position2 . "</position>\n";
				$xml_out .= "\t\t\t<alleles>" . $self->{all_snp_hash}{$position_name2} . "</alleles>\n";
				$xml_out .= "\t\t</secondary_snp>\n";
			}
			elsif(exists $self->{target_snp_hash}{$position_name2}) {
				$self->{gen_out} = $self->{gen_out} . "{" . $self->{target_snp_hash}{$position_name2} . "}";
				my ($allele1, $allele2) = split(/,/, $self->{target_snp_hash}{$position_name2} );
				
				$xml_out .= "\t\t<secondary_snp>\n";
				$xml_out .= "\t\t\t<position>" . $position2 . "</position>\n";
				$xml_out .= "\t\t\t<alleles>" . $self->{target_snp_hash}{$position_name2} . "</alleles>\n";
				$xml_out .= "\t\t</secondary_snp>\n";
			}
			else{
				$self->{gen_out} .= $value;
			}
		}
		
		$xml_out .= "\t</polymorphisms>\n";
		$xml_out .= "\t<adf>";
		$self->{gen_out} = $xml_out . $self->{gen_out} . "\t</adf>\n</fragment>\n</msqtadf>\n";
	}
	
	# delete dashes outside of snps
	$self->{gen_out} =~ s/([A-Z])-+?([A-Z])/$1$2/g;
	# highlight
# 	$self->{gen_out} =~ s/(\[.+?\])/<font color="red"><b>$1<\/b><\/font>/g;
# 	$self->{gen_out} =~ s/(\{.+?\})/<font color="blue"><b>$1<\/b><\/font>/g;
}

1;
